﻿using System;
using System.Collections.Generic;
using System.Linq;
using ff14bot;
using ff14bot.Enums;
using ff14bot.Interfaces;
using ff14bot.Managers;
using ff14bot.Objects;
using System.Threading.Tasks;
using System.Threading;

namespace Sneaky
{
    public class Sneaky : IBotPlugin
    {
        internal SLogger Logger { get; private set; }
        internal List<GameObject> NearbyUnits;
        internal List<string> AvoidUnits;
        internal int StealthTrigger = 25;
        private Settings _settingsWindow;
        private DateTime cooldownTimer;


        private Sneaky()
        {

        }

        #region Plugin Information

        public string Author
        {
            get { return " hueCannibal"; }
        }

        public string Name
        {
            get { return "Sneaky"; }
        }

        public string Description
        {
            get { return "Enter Stealth on enemy threat."; }
        }

        public Version Version
        {
            get { return new Version(0, 0, 3); }
        }


        public bool WantButton
        {
            get { return true; }
        }

        public string ButtonText
        {
            get { return "Sneaky"; }
        }

        public bool Equals(IBotPlugin other)
        {
            throw new NotImplementedException();
        }

        #endregion

        public void OnButtonPress()
        {
            if (_settingsWindow == null || _settingsWindow.IsDisposed || _settingsWindow.Disposing)
            {
                    _settingsWindow = new Settings(this);
                    _settingsWindow.Show();
            }
            else
            {
                _settingsWindow.BringToFront();
            }
        }

        public void OnInitialize()
        {
            if (Logger == null)
                Logger = new SLogger();

            NearbyUnits = new List<GameObject>();
            AvoidUnits = new List<string>();
            if(cooldownTimer == null)
            {
                cooldownTimer = DateTime.Now;
            }
        }


        private void DetectRoutine()
        {
            if(PlayerNearEnemy())
            {
                CastStealth();
            }
            else
            {
                CancelStealth();
            }
        }

        public void OnPulse()
        {

            //if(TreeRoot.Current.EnglishName != "Gathering")
            //  return;
            
            if(Core.Me.InCombat || Core.Me.IsCasting || Core.Me.IsDead)
                return;

            if (!IsValidDolClass(Core.Me.CurrentJob))
                return;

            // Scan for threats in a one second interval
            TimeSpan cooldownDiff = DateTime.Now - cooldownTimer;
            if(cooldownDiff.TotalSeconds >= 1)
            {
                DetectRoutine();
                cooldownTimer = DateTime.Now;
            }
        }


        public void OnShutdown()
        {
        }

        public void OnDisabled()
        {
        }


        public void OnEnabled()
        {
        }


        /// <summary>
        /// Returns True if the current job is set to either Miner, Fisher or Botanist.
        /// </summary>
        /// <param name="job"></param>
        /// <returns>True if the passed job is a gathering job, False otherwise.</returns>
        internal bool IsValidDolClass(ClassJobType job)
        {
            bool retBool;
            switch (job)
            {
                case ClassJobType.Miner:
                case ClassJobType.Fisher:
                case ClassJobType.Botanist:
                    retBool = true;
                    break;
                default:
                    retBool = false;
                    break;
            }

            return retBool;
        }

        internal List<GameObject> GetNearUnits(int range=90)
        {
            var units = GameObjectManager.GameObjects;
            var retUnits = new List<GameObject>();
            foreach(var unit in units.OrderBy(r => r.Distance()).Where(t => t.IsMe == false && t.Type == GameObjectType.BattleNpc && t.Distance() <= range))
            {
                if (!string.IsNullOrEmpty(unit.EnglishName))
                    retUnits.Add(unit);
            }

            return retUnits;
        }


        internal bool IsStealthActive()
        {
            return Core.Me.Auras.Any(aura => aura.Name == "Stealth");
        }


        internal void CastStealth()
        {
            if(!Actionmanager.HasSpell("Stealth") || Core.Me.IsDead)
                return;
            // Dismount to enter stealth
            var curPoi = ff14bot.Helpers.Poi.Current;
            if(curPoi.Type == ff14bot.Helpers.PoiType.Gather)
            {
                if(Core.Me.Distance(curPoi.Location) <= StealthTrigger + 20 && Core.Me.IsMounted)
                {
                    Actionmanager.Dismount();
                }
            }
            if(!Actionmanager.CanCast("Stealth", null))
                return;
            if(!IsStealthActive())
                Actionmanager.DoAction("Stealth", Core.Me);
        }


        internal void CancelStealth()
        {
            if(!Actionmanager.CanCast("Stealth", null))
                return;

            if(IsStealthActive())
                Actionmanager.DoAction("Stealth", Core.Me);
        }


        internal bool PlayerNearEnemy()
        {
            foreach(string avoidUnitName in AvoidUnits.Where(u => u != null))
            {
                var currentNearbyUnits = GetNearUnits(StealthTrigger);
                if (currentNearbyUnits.Any())
                {
                    if(currentNearbyUnits.Where(u => u.EnglishName == avoidUnitName && !u.IsMe && u.Type == GameObjectType.BattleNpc && u.Distance(Core.Me) <= StealthTrigger).Any())
                    {
                        return true;
                    }
                }
            }
            return false;
        }


    }
}